@@ -5,7 +5,7 @@ use 5.005; # 5.004 seems to have problems with use base
 use vars qw( @ISA $AUTOLOAD );
 use Carp;
 
-our $VERSION = '3.13';
+our $VERSION = '3.15';
 our $DEBUG = 0;
 
 use HTTP::OAI::UserAgent;
@@ -28,21 +28,20 @@ use HTTP::OAI::Set;
 sub new {
 	my ($class,%args) = @_;
 	my %ARGS = %args;
-	for(qw(baseURL resume repository handlers)) {
-		delete $ARGS{$_};
-	}
+	delete @ARGS{qw(baseURL resume repository handlers onRecord)};
 	my $self = $class->SUPER::new(%ARGS);
 
 	$self->{'resume'} = exists($args{resume}) ? $args{resume} : 1;
 	$DEBUG = $args{debug};
 	$self->{'handlers'} = $args{'handlers'};
+	$self->{'onRecord'} = $args{'onRecord'};
 	$self->agent('OAI-PERL/'.$VERSION);
 
 	# Record the base URL this harvester instance is associated with
 	$self->{repository} =
 		$args{repository} ||
 		HTTP::OAI::Identify->new(baseURL=>$args{baseURL});
-	croak "Requires repository or baseURL" unless $self->repository && $self->repository->baseURL;
+	croak "Requires repository or baseURL" unless $self->repository and $self->repository->baseURL;
 	# Canonicalise
 	$self->baseURL($self->baseURL);
 
@@ -93,127 +92,134 @@ sub AUTOLOAD {
 	my $self = shift;
 	my $name = $AUTOLOAD;
 	$name =~ s/.*:://;
-#	warn join(',',map { ref($_) || $_ } @_);
-	if($name =~ /GetRecord|Identify|ListIdentifiers|ListMetadataFormats|ListRecords|ListSets/) {
-		my %args = (
-			verb=>$name,
-			@_
-		);
-		my $handlers = $args{handlers}||$self->{'handlers'};
-		delete $args{handlers};
-		if( !$args{force} &&
-			defined($self->repository->version) &&
-			'2.0' eq $self->repository->version &&
-			(my @errors = HTTP::OAI::Repository::validate_request(%args)) ) {
-			return new HTTP::OAI::Response(
-				code=>503,
-				message=>'Invalid Request (use \'force\' to force a non-conformant request): ' . $errors[0]->toString,
-				errors=>\@errors
-			);
-		}
-		delete $args{force};
-		for( keys %args ) {
-			delete $args{$_} if !defined($args{$_}) || !length($args{$_});
-		}
-	
-		# Check for a static repository (sets _static)
-		if( !$self->{_interogated} ) {
-			$self->interogate();
-			$self->{_interogated} = 1;
-		}
-		
-		if( 'ListIdentifiers' eq $name &&
-			defined($self->repository->version) && 
-			'1.1' eq $self->repository->version ) {
-			delete $args{metadataPrefix};
-		}
 
-		my $r = "HTTP::OAI::$name"->new(
-			harvestAgent=>$self,
-			handlers=>$handlers,
-		);
-		$r->headers->{_args} = \%args;
-
-		# Parse all the records if _static set
-		if( defined($self->{_static}) && !defined($self->{_records}) ) {
-			my $lmdf = HTTP::OAI::ListMetadataFormats->new(
-				handlers=>$handlers,
-			);
-			$lmdf->headers->{_args} = {
-				%args,
-				verb=>'ListMetadataFormats',
-			};
-			# Find the metadata formats
-			$lmdf = $lmdf->parse_string($self->{_static});
-			return $lmdf unless $lmdf->is_success;
-			@{$self->{_formats}} = $lmdf->metadataFormat;
-			# Extract all records
-			$self->{_records} = {};
-			for($lmdf->metadataFormat) {
-				my $lr = HTTP::OAI::ListRecords->new(
-					handlers=>$handlers,
-				);
-				$lr->headers->{_args} = {
-					%args,
-					verb=>'ListRecords',
-					metadataPrefix=>$_->metadataPrefix,
-				};
-				$lr->parse_string($self->{_static});
-				@{$self->{_records}->{$_->metadataPrefix}} = $lr->record;
-			}
-			undef($self->{_static});
-		}
-		
-		# Make the remote request and return the result
-		if( !defined($self->{_records}) ) {
-			return $self->request({baseURL=>$self->baseURL,%args},undef,undef,undef,$r);
-		# Parse our memory copy of the static repository
-		} else {
-			$r->code(200);
-			# Format doesn't exist
-			if( $name =~ /^GetRecord|ListIdentifiers|ListRecords$/ &&
-				!exists($self->{_records}->{$args{metadataPrefix}}) ) {
-				$r->code(600);
-				$r->errors(HTTP::OAI::Error->new(
-					code=>'cannotDisseminateFormat',
-				));
-			# GetRecord
-			} elsif( $name eq 'GetRecord' ) {
-				for(@{$self->{_records}->{$args{metadataPrefix}}}) {
-					if( $_->identifier eq $args{identifier} ) {
-						$r->record($_);
-						return $r;
-					}
-				}
-				$r->code(600);
-				$r->errors(HTTP::OAI::Error->new(
-					code=>'idDoesNotExist'
-				));
-			# Identify
-			} elsif( $name eq 'Identify' ) {
-				$r = $self->repository();
-			# ListIdentifiers
-			} elsif( $name eq 'ListIdentifiers' ) {
-				$r->identifier(map { $_->header } @{$self->{_records}->{$args{metadataPrefix}}})
-			# ListMetadataFormats
-			} elsif( $name eq 'ListMetadataFormats' ) {
-				$r->metadataFormat(@{$self->{_formats}});
-			# ListRecords
-			} elsif( $name eq 'ListRecords' ) {
-				$r->record(@{$self->{_records}->{$args{metadataPrefix}}});
-			# ListSets
-			} elsif( $name eq 'ListSets' ) {
-				$r->errors(HTTP::OAI::Error->new(
-					code=>'noSetHierarchy',
-					message=>'Static Repositories do not support sets',
-				));
-			}
-			return $r;
-		}
-	} else {
+	# Request isn't an OAI-PMH verb (must be for LWP)
+	if($name !~ /^GetRecord|Identify|ListIdentifiers|ListMetadataFormats|ListRecords|ListSets$/) {
 		my $superior = "SUPER::$name";
 		return $self->$superior(@_);
 	}
+
+	my %args = (
+		verb=>$name,
+		@_
+	);
+	my $handlers = delete($args{handlers}) || $self->{'handlers'};
+	my $onRecord = delete($args{onRecord}) || $self->{'onRecord'};
+
+	if( !$args{force} &&
+		defined($self->repository->version) &&
+		'2.0' eq $self->repository->version &&
+		(my @errors = HTTP::OAI::Repository::validate_request(%args)) ) {
+		return new HTTP::OAI::Response(
+			code=>503,
+			message=>'Invalid Request (use \'force\' to force a non-conformant request): ' . $errors[0]->toString,
+			errors=>\@errors
+		);
+	}
+
+	delete $args{force};
+	# Get rid of any empty arguments
+	for( keys %args ) {
+		delete $args{$_} if !defined($args{$_}) || !length($args{$_});
+	}
+
+	# Check for a static repository (sets _static)
+	if( !$self->{_interogated} ) {
+		$self->interogate();
+		$self->{_interogated} = 1;
+	}
+	
+	if( 'ListIdentifiers' eq $name &&
+		defined($self->repository->version) && 
+		'1.1' eq $self->repository->version ) {
+		delete $args{metadataPrefix};
+	}
+
+	my $r = "HTTP::OAI::$name"->new(
+		harvestAgent => $self,
+		handlers => $handlers,
+		onRecord => $onRecord,
+	);
+	$r->headers->{_args} = \%args;
+
+	# Parse all the records if _static set
+	if( defined($self->{_static}) && !defined($self->{_records}) ) {
+		my $lmdf = HTTP::OAI::ListMetadataFormats->new(
+			handlers => $handlers,
+			onRecord => $onRecord,
+		);
+		$lmdf->headers->{_args} = {
+			%args,
+			verb=>'ListMetadataFormats',
+		};
+		# Find the metadata formats
+		$lmdf = $lmdf->parse_string($self->{_static});
+		return $lmdf unless $lmdf->is_success;
+		@{$self->{_formats}} = $lmdf->metadataFormat;
+		# Extract all records
+		$self->{_records} = {};
+		for($lmdf->metadataFormat) {
+			my $lr = HTTP::OAI::ListRecords->new(
+				handlers => $handlers,
+				onRecord => $onRecord,
+			);
+			$lr->headers->{_args} = {
+				%args,
+				verb=>'ListRecords',
+				metadataPrefix=>$_->metadataPrefix,
+			};
+			$lr->parse_string($self->{_static});
+			@{$self->{_records}->{$_->metadataPrefix}} = $lr->record;
+		}
+		undef($self->{_static});
+	}
+	
+	# Make the remote request and return the result
+	if( !defined($self->{_records}) ) {
+		return $self->request({baseURL=>$self->baseURL,%args},undef,undef,undef,$r);
+	# Parse our memory copy of the static repository
+	} else {
+		$r->code(200);
+		# Format doesn't exist
+		if( $name =~ /^GetRecord|ListIdentifiers|ListRecords$/ &&
+			!exists($self->{_records}->{$args{metadataPrefix}}) ) {
+			$r->code(600);
+			$r->errors(HTTP::OAI::Error->new(
+				code=>'cannotDisseminateFormat',
+			));
+		# GetRecord
+		} elsif( $name eq 'GetRecord' ) {
+			for(@{$self->{_records}->{$args{metadataPrefix}}}) {
+				if( $_->identifier eq $args{identifier} ) {
+					$r->record($_);
+					return $r;
+				}
+			}
+			$r->code(600);
+			$r->errors(HTTP::OAI::Error->new(
+				code=>'idDoesNotExist'
+			));
+		# Identify
+		} elsif( $name eq 'Identify' ) {
+			$r = $self->repository();
+		# ListIdentifiers
+		} elsif( $name eq 'ListIdentifiers' ) {
+			$r->identifier(map { $_->header } @{$self->{_records}->{$args{metadataPrefix}}})
+		# ListMetadataFormats
+		} elsif( $name eq 'ListMetadataFormats' ) {
+			$r->metadataFormat(@{$self->{_formats}});
+		# ListRecords
+		} elsif( $name eq 'ListRecords' ) {
+			$r->record(@{$self->{_records}->{$args{metadataPrefix}}});
+		# ListSets
+		} elsif( $name eq 'ListSets' ) {
+			$r->errors(HTTP::OAI::Error->new(
+				code=>'noSetHierarchy',
+				message=>'Static Repositories do not support sets',
+			));
+		}
+		return $r;
+	}
 }
 
 sub interogate {
@@ -244,13 +250,13 @@ HTTP::OAI::Harvester - Agent for harvesting from Open Archives version 1.0, 1.1,
 
 =head1 DESCRIPTION
 
-HTTP::OAI::Harvester provides the front-end to the OAI-PERL library for harvesting from OAI repositories. Direct use of the other OAI-PERL modules for harvesting should be avoided.
+C<HTTP::OAI::Harvester> is the harvesting front-end in the OAI-PERL library.
 
-To harvest from an OAI-compliant repository first create the HTTP::OAI::Harvester interface using the baseURL option. It is recommended that you request an Identify from the Repository and use the repository method to update the Identify object used by the harvester.
+To harvest from an OAI-PMH compliant repository create an C<HTTP::OAI::Harvester> object using the baseURL option and then call OAI-PMH methods to request data from the repository. To handle version 1.0/1.1 repositories automatically you B<must> request C<Identify()> first.
 
-When making OAI requests the underlying L<HTTP::OAI::UserAgent|HTTP::OAI::UserAgent> module will take care of automatic redirection (http code 302) and retry-after (http code 503).
+It is recommended that you request an Identify from the Repository and use the C<repository()> method to update the Identify object used by the harvester.
 
-OAI flow control (i.e. resumption tokens) is handled transparently by HTTP::OAI::Harvester.
+When making OAI requests the underlying L<HTTP::OAI::UserAgent> module will take care of automatic redirection (http code 302) and retry-after (http code 503). OAI-PMH flow control (i.e. resumption tokens) is handled transparently by C<HTTP::OAI::Response>.
 
 =head2 Static Repository Support
 
@@ -268,9 +274,11 @@ If a static repository is found the response is cached, and further requests are
 
 You should refer to the Open Archives Protocol version 2.0 and other OAI documentation, available from http://www.openarchives.org/.
 
+Note OAI-PMH 1.0 and 1.1 are deprecated.
+
 =head1 BEFORE USING EXAMPLES
 
-In the examples I use arXiv.org's, and cogprints OAI interfaces. To avoid causing annoyance to their server administrators please contact them before performing testing or large downloads (or use other, less loaded, servers).
+In the examples I use arXiv.org's and cogprints OAI interfaces. To avoid causing annoyance to their server administrators please contact them before performing testing or large downloads (or use other, less loaded, servers for testing).
 
 =head1 SYNOPSIS
 
@@ -305,10 +313,6 @@ In the examples I use arXiv.org's, and cogprints OAI interfaces. To avoid causin
 		"requestURL => ", $response->requestURL, "\n";
 
 	while( my $id = $response->next ) {
-		if( $id->is_error ) {
-			print "Error: ", $id->code, " (", $id->message, ")\n";
-			last;
-		}
 		print "identifier => ", $id->identifier;
 		# Only available from OAI 2.0 repositories
 		print " (", $id->datestamp, ")" if $id->datestamp;
@@ -320,26 +324,22 @@ In the examples I use arXiv.org's, and cogprints OAI interfaces. To avoid causin
 		}
 	}
 
+	# Using a handler
 	$response = $h->ListRecords(
 		metadataPrefix=>'oai_dc',
 		handlers=>{metadata=>'HTTP::OAI::Metadata::OAI_DC'},
 	);
-	if( $response->is_error ) {
-		print "Error: ", $response->code,
-			" (", $response->message, ")\n";
-		exit();
-	}
-
 	while( my $rec = $response->next ) {
-		if( $rec->is_error ) {
-			die $rec->message;
-		}
 		print $rec->identifier, "\t",
 			$rec->datestamp, "\n",
 			$rec->metadata, "\n";
 		print join(',', @{$rec->metadata->dc->{'title'}}), "\n";
 	}
+	if( $rec->is_error ) {
+		die $response->message;
+	}
 
+	# Offline parsing
 	$I = HTTP::OAI::Identify->new();
 	$I->parse_string($content);
 	$I->parse_file($fh);
@@ -350,9 +350,9 @@ In the examples I use arXiv.org's, and cogprints OAI interfaces. To avoid causin
 
 =item HTTP::OAI::Harvester->new( %params )
 
-This constructor method returns a new instance of HTTP::OAI::Harvester. Requires either an L<HTTP::OAI::Identify|HTTP::OAI::Identify> object, which in turn must contain a baseURL, or a baseURL from which to construct an Identify object.
+This constructor method returns a new instance of C<HTTP::OAI::Harvester>. Requires either an L<HTTP::OAI::Identify> object, which in turn must contain a baseURL, or a baseURL from which to construct an Identify object.
 
-Any other parameters are passed to the L<HTTP::OAI::UserAgent|HTTP::OAI::UserAgent> module, and from there to the L<LWP::UserAgent|LWP::UserAgent> module.
+Any other parameters are passed to the L<HTTP::OAI::UserAgent> module, and from there to the L<LWP::UserAgent> module.
 
 	$h = HTTP::OAI::Harvester->new(
 		baseURL	=>	'http://arXiv.org/oai2',
@@ -368,31 +368,41 @@ Any other parameters are passed to the L<HTTP::OAI::UserAgent|HTTP::OAI::UserAge
 
 =item $h->repository()
 
-Returns and optionally sets the HTTP::OAI::Identify data used by the Harvester agent.
+Returns and optionally sets the L<HTTP::OAI::Identify> object used by the Harvester agent.
 
-=item $h->resume()
+=item $h->resume( [1] )
 
-If set to true (default) resumption tokens will automatically be handled by requesting the next partial list.
+If set to true (default) resumption tokens will automatically be handled by requesting the next partial list during C<next()> calls.
 
 =back
 
-=head2 OAI-PMH Verbs
+=head1 OAI-PMH Verbs
 
-These methods return either a:
+The 6 OAI-PMH Verbs are the requests supported by an OAI-PMH interface.
 
-L<HTTP::Response|HTTP::Response>
+=head2 Error Messages
 
-If there was a problem making the HTTP request, or a module subclassed from L<HTTP::OAI::Response|HTTP::OAI::Response>, corresponding to the method name (e.g. GetRecord will return L<HTTP::OAI::GetRecord|HTTP::OAI::GetRecord>).
+Use C<is_success()> or C<is_error()> on the returned object to determine whether an error occurred (see L<HTTP::OAI::Response>).
 
-Use $r->is_success to determine whether an error occurred.
+C<code()> and C<message()> return the error code (200 is success) and a human-readable message respectively. L<Errors|HTTP::OAI::Error> returned by the repository can be retrieved using the C<errors()> method:
 
-Use $r->code and $r->message to obtain the error code and a human-readable message. OAI level errors can be retrieved using the $r->errors method.
+	foreach my $error ($r->errors) {
+		print $error->code, "\t", $error->message, "\n";
+	}
 
-If the response contained an L<HTTP::OAI::ResumptionToken|HTTP::OAI::ResumptionToken> this can be retrieved using the $r->resumptionToken method. When enumerating through ListIdentifiers, ListRecords or ListSets you should check $rec->is_success as an error may have occurred while attempted to retrieve the next partial list of matches.
+Note: C<is_success()> is true for the OAI Error Code C<noRecordsMatch> (i.e. empty set), although C<errors()> will still contain the OAI error.
+
+=head2 Flow Control
+
+If the response contained a L<resumption token|HTTP::OAI::ResumptionToken> this can be retrieved using the $r->resumptionToken method.
+
+=head2 Methods
+
+These methods return an object subclassed from L<HTTP::Response> (where the class corresponds to the verb requested, e.g. C<GetRecord> requests return an C<HTTP::OAI::GetRecord> object).
 
 =over 4
 
-=item $h->GetRecord( %params )
+=item $r = $h->GetRecord( %params )
 
 Get a single record from the repository identified by identifier, in format metadataPrefix.
 
@@ -400,20 +410,19 @@ Get a single record from the repository identified by identifier, in format meta
 		identifier	=>	'oai:arXiv:hep-th/0001001', # Required
 		metadataPrefix	=>	'oai_dc' # Required
 	);
-	die $gr->message if $gr->is_error;
 	$rec = $gr->next;
 	die $rec->message if $rec->is_error;
 	printf("%s (%s)\n", $rec->identifier, $rec->datestamp);
 	$dom = $rec->metadata->dom;
 
-=item $h->Identify()
+=item $r = $h->Identify()
 
 Get information about the repository.
 
 	$id = $h->Identify();
 	print join ',', $id->adminEmail;
 
-=item $h->ListIdentifiers( %params )
+=item $r = $h->ListIdentifiers( %params )
 
 Retrieve the identifiers, datestamps, sets and deleted status for all records within the specified date range (from/until) and set spec (set). 1.x repositories will only return the identifier. Or, resume an existing harvest by specifying resumptionToken.
 
@@ -423,26 +432,25 @@ Retrieve the identifiers, datestamps, sets and deleted status for all records wi
 		until		=>		'2001-10-31',
 		set=>'physics:hep-th',
 	);
-	die $lr->message if $lr->is_error;
 	while($rec = $lr->next)
 	{
-		die $rec->message if $rec->is_error;
 		{ ... do something with $rec ... }
 	}
+	die $lr->message if $lr->is_error;
 
-=item $h->ListMetadataFormats( %params )
+=item $r = $h->ListMetadataFormats( %params )
 
 List available metadata formats. Given an identifier the repository should only return those metadata formats for which that item can be disseminated.
 
 	$lmdf = $h->ListMetadataFormats(
 		identifier => 'oai:arXiv.org:hep-th/0001001'
 	);
-	die $lmdf->message if $lmdf->is_error;
 	for($lmdf->metadataFormat) {
 		print $_->metadataPrefix, "\n";
 	}
+	die $lmdf->message if $lmdf->is_error;
 
-=item $h->ListRecords( %params )
+=item $r = $h->ListRecords( %params )
 
 Return full records within the specified date range (from/until), set and metadata format. Or, specify a resumption token to resume a previous partial harvest.
 
@@ -452,29 +460,25 @@ Return full records within the specified date range (from/until), set and metada
 		until	=>	'2001-10-01',
 		set		=>	'physics:hep-th',
 	);
-	die $lr->message if $lr->is_error;
 	while($rec = $lr->next)
 	{
-		die $rec->message if $rec->is_error;
 		{ ... do something with $rec ... }
 	}
+	die $lr->message if $lr->is_error;
 
 =item $r = $h->ListSets( %params )
 
 Return a list of sets provided by the repository. The scope of sets is undefined by OAI-PMH, so therefore may represent any subset of a collection. Optionally provide a resumption token to resume a previous partial request.
 
 	$ls = $h->ListSets();
-	die $ls->message if $ls->is_error;
 	while($set = $ls->next)
 	{
-		die $set->message if $sec->is_error;
 		print $set->setSpec, "\n";
 	}
+	die $ls->message if $ls->is_error;
 
 =back
 
-=head1 ABOUT
+=head1 AUTHOR
 
 These modules have been written by Tim Brody E<lt>tdb01r@ecs.soton.ac.ukE<gt>.
-
-You can find links to this and other OAI tools (perl, C++, java) at: http://www.openarchives.org/tools/tools.html.
