@@ -48,7 +48,7 @@ sub generate {
 sub end_element {
 	my ($self,$hash) = @_;
 	$self->SUPER::end_element($hash);
-	my $elem = lc($hash->{Name});
+	my $elem = lc($hash->{LocalName});
 	if( $elem eq 'metadataprefix' ) {
 		$self->metadataPrefix($hash->{Text});
 	} elsif( $elem eq 'schema' ) {
